__title__ = 'rtv-downloader'
__description__ = 'Command-line program to download videos from various sites.'
__url__ = 'https://github.com/radzak/rtv-downloader'
__version__ = '2.0.7'
__author__ = 'Radek Krzak'
__author_email__ = 'radek.krzak@gmail.com'
__license__ = 'MIT'
__copyright__ = 'Copyright 2018 Radek Krzak'
